CREATE TABLE IF NOT EXISTS `client` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `uuid` binary(16) NOT NULL,
  `personId` int(10) unsigned NOT NULL,
  `name` varchar(32) COLLATE utf8_unicode_ci NOT NULL,
  `token` char(16) COLLATE utf8_unicode_ci DEFAULT NULL,
  `tokenValid` int(10) unsigned DEFAULT NULL,
  `secret` char(64) COLLATE utf8_unicode_ci DEFAULT NULL,
  `connectDate` bigint(20) unsigned DEFAULT NULL,
  PRIMARY KEY (`id`),
  KEY `client_personId_fk` (`personId`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `config` (
  `name` varchar(24) COLLATE utf8_unicode_ci NOT NULL,
  `value` text COLLATE utf8_unicode_ci DEFAULT NULL,
  PRIMARY KEY (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `deletion_event` (
  `personId` int(10) unsigned NOT NULL,
  `uuid` binary(16) NOT NULL,
  `entity` enum('object','rule') COLLATE utf8_unicode_ci NOT NULL,
  `date` bigint(20) unsigned NOT NULL,
  UNIQUE KEY `deletion_uuid` (`uuid`),
  KEY `deletion_personFk` (`personId`),
  KEY `deletion_date` (`date`),
  KEY `deletion_entity` (`entity`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `group` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `name` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `log` (
  `personId` int(10) unsigned DEFAULT NULL,
  `source` enum('application','authentication','database','sync','webServer') COLLATE utf8_unicode_ci NOT NULL,
  `level` enum('error','warning','info') COLLATE utf8_unicode_ci NOT NULL,
  `message` text COLLATE utf8_unicode_ci NOT NULL,
  `dateMilli` bigint(20) unsigned NOT NULL,
  KEY `log_personFk` (`personId`),
  KEY `log_dateMilli` (`dateMilli`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `object` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `uuid` binary(16) NOT NULL,
  `packId` int(10) unsigned DEFAULT NULL,
  `status` enum('created','active','deleted') COLLATE utf8_unicode_ci NOT NULL,
  `type` enum('file','url','pack') COLLATE utf8_unicode_ci NOT NULL,
  PRIMARY KEY (`id`),
  KEY `object_packFk` (`packId`),
  UNIQUE KEY `object_uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `object_event` (
  `id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
  `uuid` binary(16) NOT NULL,
  `objectId` int(10) unsigned NOT NULL,
  `personId` int(10) unsigned NOT NULL,
  `clientId` int(10) unsigned DEFAULT NULL,
  `ruleId` int(10) unsigned DEFAULT NULL,
  `type` enum('object','file','name','extension','url','tags','pack','rule') COLLATE utf8_unicode_ci NOT NULL,
  `event` enum('CREATED','ADDED','REMOVED','CHANGED','DELETED','RECOVERED') COLLATE utf8_unicode_ci NOT NULL,
  `size` int(10) unsigned DEFAULT NULL,
  `version` int(10) unsigned DEFAULT NULL,
  `metaValue` text COLLATE utf8_unicode_ci DEFAULT NULL,
  `date` bigint(20) unsigned NOT NULL,
  PRIMARY KEY (`id`),
  KEY `objectEvent_objectFk` (`objectId`),
  KEY `objectEvent_personFk` (`personId`),
  KEY `objectEvent_date` (`date`),
  KEY `objectEvent_clientFk` (`clientId`),
  UNIQUE KEY `objectEvent_uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `object_rule` (
  `objectId` int(10) unsigned NOT NULL,
  `ruleId` int(10) unsigned NOT NULL,
  PRIMARY KEY (`objectId`,`ruleId`),
  KEY `objectRule_ruleFk` (`ruleId`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `person` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `uuid` binary(16) NOT NULL,
  `forename` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  `surname` varchar(64) COLLATE utf8_unicode_ci NOT NULL,
  `active` tinyint(3) unsigned NOT NULL,
  `admin` tinyint(3) unsigned NOT NULL,
  `changeDate` bigint(20) unsigned NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `person_uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `person_group` (
  `personId` int(10) unsigned NOT NULL,
  `groupId` int(10) unsigned NOT NULL,
  PRIMARY KEY (`personId`,`groupId`),
  KEY `personGroup_groupFk` (`groupId`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `rule` (
  `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
  `uuid` binary(16) NOT NULL,
  `name` varchar(32) COLLATE utf8_unicode_ci NOT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `rule_uuid` (`uuid`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `rule_effect` (
  `ruleId` int(10) unsigned NOT NULL,
  `personId` int(10) unsigned NOT NULL,
  `newAccess` tinyint(3) unsigned NOT NULL,
  `rights` tinyint(3) unsigned NOT NULL,
  `changeDate` bigint(20) unsigned NOT NULL,
  KEY `ruleEffect_personId` (`personId`),
  KEY `ruleEffect_ruleId` (`ruleId`),
  KEY `ruleEffect_newAccess` (`newAccess`),
  KEY `ruleEffect_changeDate` (`changeDate`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `rule_group` (
  `ruleId` int(10) unsigned NOT NULL,
  `groupId` int(10) unsigned NOT NULL,
  `rights` tinyint(3) unsigned NOT NULL,
  PRIMARY KEY (`ruleId`,`groupId`),
  KEY `ruleGroup_groupFk` (`groupId`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

CREATE TABLE IF NOT EXISTS `rule_person` (
  `ruleId` int(10) unsigned NOT NULL,
  `personId` int(10) unsigned NOT NULL,
  `rights` tinyint(3) unsigned NOT NULL,
  PRIMARY KEY (`ruleId`,`personId`),
  KEY `rulePerson_personFk` (`personId`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

ALTER TABLE `client`
  ADD CONSTRAINT `client_personFk` FOREIGN KEY (`personId`) REFERENCES `person` (`id`);

ALTER TABLE `log`
  ADD CONSTRAINT `log_personFk` FOREIGN KEY (`personId`) REFERENCES `person` (`id`);

ALTER TABLE `object`
  ADD CONSTRAINT `object_packFk` FOREIGN KEY (`packId`) REFERENCES `object` (`id`);

ALTER TABLE `object_event`
  ADD CONSTRAINT `objectEvent_clientFk` FOREIGN KEY (`clientId`) REFERENCES `client` (`id`),
  ADD CONSTRAINT `objectEvent_objectFk` FOREIGN KEY (`objectId`) REFERENCES `object` (`id`),
  ADD CONSTRAINT `objectEvent_personFk` FOREIGN KEY (`personId`) REFERENCES `person` (`id`),
  ADD CONSTRAINT `objectEvent_ruleFk` FOREIGN KEY (`ruleId`) REFERENCES `rule` (`id`);

ALTER TABLE `object_rule`
  ADD CONSTRAINT `objectRule_objectFk` FOREIGN KEY (`objectId`) REFERENCES `object` (`id`),
  ADD CONSTRAINT `objectRule_ruleFk` FOREIGN KEY (`ruleId`) REFERENCES `rule` (`id`);

ALTER TABLE `person_group`
  ADD CONSTRAINT `personGroup_groupFk` FOREIGN KEY (`groupId`) REFERENCES `group` (`id`),
  ADD CONSTRAINT `personGroup_personFk` FOREIGN KEY (`personId`) REFERENCES `person` (`id`);

ALTER TABLE `rule_effect`
  ADD CONSTRAINT `ruleEffect_personId` FOREIGN KEY (`personId`) REFERENCES `person` (`id`),
  ADD CONSTRAINT `ruleEffect_ruleId` FOREIGN KEY (`ruleId`) REFERENCES `rule` (`id`);

ALTER TABLE `rule_group`
  ADD CONSTRAINT `ruleGroup_groupFk` FOREIGN KEY (`groupId`) REFERENCES `group` (`id`),
  ADD CONSTRAINT `ruleGroup_ruleFk` FOREIGN KEY (`ruleId`) REFERENCES `rule` (`id`);

ALTER TABLE `rule_person`
  ADD CONSTRAINT `rulePerson_personFk` FOREIGN KEY (`personId`) REFERENCES `person` (`id`),
  ADD CONSTRAINT `rulePerson_ruleFk` FOREIGN KEY (`ruleId`) REFERENCES `rule` (`id`);

INSERT IGNORE INTO `config` (`name`, `value`) VALUES
	('appVersion', '0.9'),
	('cleanLogDays', '30'),
	('cleanObjectsDeletedDays', '30'),
	('cleanObjectsVersionCount', '5'),
	('logLevelApp', '1'),
	('logLevelAuth', '1'),
	('logLevelDb', '1'),
	('logLevelSync', '1'),
	('logLevelWww', '1'),
	('serverTitle', 'allTags Server'),
	('syncCheckIntervalSeconds', '3'),
	('wwwUploadLimitMb', '512');